util = require "util"
local player = require "classes.player"
local Vector3 = require "types.vector3"
local List = require "types.list"

local Feed = require "classes.feed"
local feed = List()

local Duck = require "classes.duck"
local ducks = List{Duck.new(Vector3(0, 0.4, 3))}

local function create_feed(position, direction)
  local f = Feed.new(position, direction)
  feed:push(f)
  local eligible_ducks = ducks:filter(
    function (duck)
      return duck.state ~= Duck.STATES.CHASE
    end
  )
  if #eligible_ducks == 0 then return end
  eligible_ducks[1]:start_chase(f)
  f.occupied = true
end

local function delete_feed(f)
  util.list_remove_value(feed, f)
end

-- called every frame, with constant delta time
function game_tick()
  -- ctx.initialization_needed is true first frame and every time dynamic reload is performed
  if ctx.initialization_needed then
    -- ctx.udata persists on reload
    ctx.udata = {
      capture = false,
    }
    player.ThrowPressed:connect(create_feed)
    for _, v in ipairs(ducks) do
      v.AteFeed:connect(delete_feed)
    end
  end

  ctx.mouse_capture = ctx.udata.capture
  input_action{name = "toggle_mouse", control = "ESCAPE"}

  if input_action_just_pressed{name = "toggle_mouse"} then
    ctx.udata.capture = not ctx.udata.capture
  end

  for _, v in ipairs(feed) do
    v:tick(ctx)
  end

  for _, v in ipairs(ducks) do
    v:tick(ctx)
  end

  player:tick(ctx)
  local q = util.create_plane_quad(Vector3(0, 0, 0), Vector3.UP, 20)
  local params = {
    texture = "images/measure001a.png",
    texture_region = { x = 0, y = 0, w = 512, h = 512 },
  }
  draw_quad(util.merge(q, params))
end
